@@ -43,6 +43,7 @@ import urllib3
 import getpass
 import shutil
 import stat
+import urllib.parse
 
 import logging
 
@@ -67,7 +68,7 @@ import logging
 #  Copy 'run.py' and 'settings.ini' files wherever you want to have your record repository.
 #  Windows let's you double click 'run.py'. Can run from desktop.
 # Magic number 42 is used to connect to all radios found in 'settings.ini'.
-print('0.6.68')
+print('0.6.8')
 
 # https://www.scrapehero.com/how-to-fake-and-rotate-user-agents-using-python-3/
 proxy_support = urllib.request.ProxyHandler({})
@@ -222,6 +223,7 @@ class GIni(GBase):
                 print('\t>> %-20s <<' % _)
 
             print(' \n Radio stations in your list. ')
+            print(' Please use "Ctrl + C" to stop. ')
         return
 
     @staticmethod
@@ -524,7 +526,7 @@ class GRecorder:
     @staticmethod  # copy keep-alive timeout=3000
     def thread_pull_song_name(url, ini_key):
         # print(f' thread_pull_song_name: {ini_key}')
-        update_terminal = False
+        update_terminal = ''.encode('utf-8')
         stream_song_name = GRecorder.current_song_dict[ini_key]
         GIni.fail_meta_dict[url] = 'False'
         headers = {'Connection': 'keep-alive'}
@@ -556,15 +558,26 @@ class GRecorder:
             #     stream_song_name = '_no-name-record_no_split_'
             #     # print(' _is_no_meta_data_avail ' + stream_song_name)
             #
-            # if not len(stream_song_name) <= 2:
-            #     #  GRecorder.current_song_dict[ini_key] = stream_song_name  # WRITE SONG NAME   # :DEACTIVATED:
-            #     stream_song_name = GRecorder.current_song_dict[ini_key]
+            if not len(stream_song_name) <= 2:
+                #  GRecorder.current_song_dict[ini_key] = stream_song_name  # WRITE SONG NAME   # :DEACTIVATED:
+                stream_song_name = GRecorder.current_song_dict[ini_key]                         # :REVERSE:
+
+                if not update_terminal == stream_song_name:
+                    update_terminal = stream_song_name
+                    try:
+                        print('\t\t title on ' + ini_key + ':\t' + update_terminal)
+                        # see what happens. looks like a unit test case
+                    except UnicodeEncodeError:
+                        print('\t\t title on ' + ini_key + ':\t' + 'unicode error (check "$ localectl status")')
+                        print('\t\t title on ' + ini_key + ':\t' + 'consider to add additional language support.')
+
+                    except Exception as ex:
+                        print('\t\t title on ' + ini_key + ':\t' + 'unknown error (print to your terminal)')
+                        print(ex)
+                    else:
+                        pass
+            # give recorder command to record, if we got a match from search gui
             #
-            #     if not update_terminal == stream_song_name:
-            #         update_terminal = stream_song_name
-            #         print('\t\t title on ' + ini_key + ':\t' + update_terminal)
-            # give recorder command to record, we got a match from search
-            # # GIni.search_title_keys_list.append(42)     # test value
             try:
                 for _ in GIni.search_title_keys_list:
                     if _ == ini_key:
@@ -846,6 +859,8 @@ class GRecorder:
 
                                 if os.path.exists(ghetto_recorder):
                                     os.remove(ghetto_recorder)
+                            except UnicodeEncodeError:
+                                print(f' {key} copy last file to folder (marked: _incomplete_):')
                             except Exception as ex:
                                 print(ex)
                             break
@@ -873,7 +888,7 @@ class GRecorder:
 
                 if time() - old_time > rand_short:
                     old_time = time()
-                    rh = response.headers
+                    rh = response.headers   # keep conn. alive
                     # print(response.headers)
 
                 if time() - old_time > rand_long:  # new connection
@@ -898,17 +913,23 @@ class GRecorder:
                     title_info = metadata_content.split(";")
                     title_info = title_info[0].split("=")
                     title_info = GBase.remove_special_chars(title_info[1])
-                    request.release_conn()
+                    request.release_conn()  # REQUEST not response
                     # print(f' -> {key} + {title_info}')
                 except Exception as ex:
                     print(ex)
                 else:
-                    print(f' -> {key} + {title_info}')
-                    GRecorder.current_song_dict[key] = title_info
-                    for sec in range(rand_short):
-                        if GBase.exit_app:
-                            break
-                        sleep(1)
+                    # print(f' -> {key} + {title_info}')
+                    try:
+                        GRecorder.current_song_dict[key] = title_info
+                    except Exception as ex:
+                        print(ex)
+                        pass
+                    else:
+                        for sec in range(rand_short):
+                            if GBase.exit_app:
+                                print(f' out {key} reader')
+                                break
+                            sleep(1)
 
     @staticmethod
     def fire_get_metadata(url, key):
@@ -975,7 +996,7 @@ class GRecorder:
 
                         fresh_file = directory_save + '//' + clean_name + stream_suffix
                         GRecorder.path_to_song_dict[radio_short_key] = fresh_file  # new file  with PATH!!!
-                        print(f'GRecorder.current_song_dict  {GRecorder.current_song_dict[radio_short_key]}')
+                        # print(f'GRecorder.current_song_dict  {GRecorder.current_song_dict[radio_short_key]}')
                         while not GBase.exit_app:  # inner loop get the stream and write into file
 
                             stream_song_name_path_new = GRecorder.path_to_song_dict[radio_short_key]  # look what's new
@@ -1204,10 +1225,13 @@ def record(ini_keys):
     signal.signal(signal.SIGINT, signal_handler)
 
     for ini_key in ini_keys:
-        # try:
-        stream_suffix = GIni.srv_param_dict[ini_key + '_file']  # quick fix
-        # except KeyError:
-        # stream_suffix = '.mp3'
+        stream_suffix = '.mp3'                                      # fix for 50%, lol
+        try:
+            stream_suffix = GIni.srv_param_dict[ini_key + '_file']  # sometimes malformed content-type written?
+        except KeyError:
+            stream_suffix = '.mp3'
+        except Exception as ex:
+            pass
 
         GRecorder.current_song_dict[ini_key] = '_no-name-record_no_split_'  # init the dict for this thread
         GIni.start_stop_recording[ini_key] = 'start'
